@@ -92,10 +92,11 @@ impl Term {
             Var(_) => (),
             Abs(_) => self.unabs_ref_mut().unwrap().beta_once(),
             App(_, _) => {
-                let copy = self.clone();
-                if let Ok(result) = copy.eval() {
-                    if SHOW_REDUCTIONS { println!("    {} reduces to {}", self, result) }
-                    *self = result
+                if self.lhs_ref().unwrap().unabs_ref().is_ok() {
+                    let copy = self.clone();
+                    let reduced = copy.eval().unwrap();
+                    if SHOW_REDUCTIONS { println!("    {} reduces to {}", self, reduced) };
+                    *self = reduced;
                 } else {
                     self.lhs_ref_mut().unwrap().beta_once();
                     self.rhs_ref_mut().unwrap().beta_once()
@@ -117,14 +118,11 @@ impl Term {
     /// assert_eq!(pred_one, 0.into());
     /// ```
     pub fn beta_full(&mut self) {
-        if SHOW_REDUCTIONS { println!("reducing {}", self) }
-        let mut tmp = self.clone();
-        self.beta_once();
-
-        while tmp != *self {
-            tmp = self.clone();
+        loop {
             if SHOW_REDUCTIONS { println!("reducing {}", self) }
+            let tmp = self.clone();
             self.beta_once();
+            if *self == tmp { break }
         }
 
         if SHOW_REDUCTIONS { println!("    doesn't reduce") }
