@@ -52,6 +52,21 @@ impl Term {
         }
     }
 
+    /// Returns a reference to a variable's index.
+    ///
+    /// # Example
+    /// ```
+    /// use lambda_calculus::term::Term::*;
+    ///
+    /// assert_eq!(Var(1).unvar_ref(), Ok(&1));
+    /// ```
+    pub fn unvar_ref(&self) -> Result<&usize, Error> {
+        match *self {
+            Var(ref n) => Ok(&n),
+            _ => Err(NotAVar)
+        }
+    }
+
     /// Consumes an abstraction and returns its underlying term.
     ///
     /// # Example
@@ -217,12 +232,11 @@ impl Term {
         if let Ok((_, rhs)) = self.unapp_ref_mut() { Ok(rhs) } else { Err(NotAnApp) }
     }
 
-    /// Applies two terms with substitution and variable update (but without reduction), consuming them
+    /// Applies `self` to another term with substitution and variable update, consuming them
     /// in the process.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::apply;
     /// use lambda_calculus::parser::parse;
     ///
     /// let lhs    = parse(&"λλ42(λ13)").unwrap();
@@ -231,10 +245,8 @@ impl Term {
     ///
     /// assert_eq!(lhs.apply(rhs), Ok(result));
     /// ```
-    pub fn apply(mut self, rhs: Term) -> Result<Term, Error> {
-        _apply(&mut self, rhs, 0);
-
-        Ok(try!(self.unabs()))
+    pub fn apply(self, rhs: Term) -> Result<Term, Error> {
+        apply(self, rhs)
     }
 
     /// Reduces an `App`lication by substitution and variable update.
@@ -249,11 +261,9 @@ impl Term {
     /// assert_eq!(app(i(), zero()).eval(), Ok(abs(abs(Var(1)))));
     /// ```
     pub fn eval(self) -> Result<Term, Error> {
-        let (mut lhs, rhs) = try!(self.unapp());
+        let (lhs, rhs) = try!(self.unapp());
 
-        _apply(&mut lhs, rhs, 0);
-
-        Ok(try!(lhs.unabs()))
+        apply(lhs, rhs)
     }
 }
 
@@ -280,8 +290,7 @@ pub fn abs(term: Term) -> Term { Abs(Box::new(term)) }
 /// ```
 pub fn app(lhs: Term, rhs: Term) -> Term { App(Box::new(lhs), Box::new(rhs)) }
 
-/// Applies two terms with substitution and variable update (but without reduction), consuming them
-/// in the process.
+/// Applies two terms with substitution and variable update, consuming them in the process.
 ///
 /// # Example
 /// ```
@@ -297,14 +306,14 @@ pub fn app(lhs: Term, rhs: Term) -> Term { App(Box::new(lhs), Box::new(rhs)) }
 pub fn apply(mut lhs: Term, rhs: Term) -> Result<Term, Error> {
     _apply(&mut lhs, rhs, 0);
 
-    Ok(try!(lhs.unabs()))
+    lhs.unabs()
 }
 
 fn _apply(lhs: &mut Term, rhs: Term, depth: usize) {
     match *lhs {
         Var(i) => if i == depth {
             *lhs = rhs; // substitute a top-level variable from lhs with rhs
-            update_free_variables(lhs, depth - 1) // update free variables' indices from rhs
+            update_free_variables(lhs, depth - 1, 0); // update indices of free variables from rhs
         } else if i > depth {
             *lhs = Var(i - 1) // decrement a free variable's index
         },
@@ -313,22 +322,22 @@ fn _apply(lhs: &mut Term, rhs: Term, depth: usize) {
         },
         App(_, _) => {
             _apply(lhs.lhs_ref_mut().unwrap(), rhs.clone(), depth);
-            _apply(lhs.rhs_ref_mut().unwrap(), rhs, depth);
+            _apply(lhs.rhs_ref_mut().unwrap(), rhs, depth)
         }
     }
 }
 
-fn update_free_variables(term: &mut Term, depth: usize) {
+fn update_free_variables(term: &mut Term, added_depth: usize, own_depth: usize) {
     match *term {
-        Var(i) => if i > depth {
-            *term = Var(i + depth)
+        Var(i) => if i > own_depth {
+            *term = Var(i + added_depth)
         },
         Abs(_) => {
-            update_free_variables(term.unabs_ref_mut().unwrap(), depth)
+            update_free_variables(term.unabs_ref_mut().unwrap(), added_depth, own_depth + 1)
         },
         App(_, _) => {
-            update_free_variables(term.lhs_ref_mut().unwrap(), depth);
-            update_free_variables(term.rhs_ref_mut().unwrap(), depth);
+            update_free_variables(term.lhs_ref_mut().unwrap(), added_depth, own_depth);
+            update_free_variables(term.rhs_ref_mut().unwrap(), added_depth, own_depth)
         }
     }
 }
@@ -369,8 +378,8 @@ mod test {
     use arithmetic::{zero, succ, pred};
     use combinators::i;
     use parser::parse;
-    use term::{apply, abs};
-    use term::Term::Var;
+    use super::{apply, abs};
+    use super::Term::Var;
 
     #[test]
     fn applying() {
@@ -379,7 +388,7 @@ mod test {
         let result = parse(&"λ3(λ61)(λ1(λ71))").unwrap();
         assert_eq!(apply(lhs, rhs), Ok(result));
 
-        assert_eq!(i().app(zero()).apply().unwrap(), abs(abs(Var(1))));
+        assert_eq!(i().app(zero()).eval().unwrap(), abs(abs(Var(1))));
     }
 
     #[test]
